from ..version import __version__

USER_AGENT = "ActiveCampaign-Python/" + __version__


class API_ENDPOINT(object):
    EMAIL_ACTIVITIES = 'emailActivities'
    MESSAGE = 'messages'
    SEGMENT = 'segments'
    ADDRESSES = 'addresses'
    CONTACT = 'contacts'
    FIELD = 'fields'
    FIELD_VALUE = 'fieldValues'
    ACCOUNTS = 'accounts'
    ACCOUNT_CONTACTS = 'accountContacts'
    ACCOUNT_CUSTOM_FIELD_META = 'accountCustomFieldMeta'
    ACCOUNT_CUSTOM_FIELD_DATA = 'accountCustomFieldData'
    WEBHOOK = 'webhooks'




from activecampaign_api.api.abstract_request import AbstractRequest
from activecampaign_api.api.consts import API_ENDPOINT
from activecampaign_api.api.custom_field import CustomFieldsRequest
from activecampaign_api.api.custom_field_value import CustomFieldValueRequest


class Attribute:
    def __init__(self, model, name):
        self.model = model
        self.name = name


class Contact(object):
    def __init__(self, contact_request, id=None, cdate=None, email=None,
                 phone=None, first_name=None,
                 last_name=None):
        self.contact_request = contact_request
        self.data = {}
        self.id = id
        self.cdate = cdate
        self.email = email
        self.phone = phone
        self.first_name = first_name
        self.last_name = last_name
        self._custom_fields = {}
        self._custom_fields_value = {}
        self._fields = {}

    def from_dict(self, data):
        self.data = data
        if 'id' in data:
            self.id = data['id']
        if 'cdate' in data:
            self.cdate = data['cdate']
        if 'email' in data:
            self.email = data['email']
        if 'phone' in data:
            self.phone = data['phone']
        if 'firstName' in data:
            self.first_name = data['firstName']
        if 'lastName' in data:
            self.last_name = data['lastName']
        return self

    def to_dict(self):
        data = {
            'cdate': self.cdate,
            'email': self.email,
            'phone': self.phone,
            'first_name': self.first_name,
            'last_name': self.last_name,
        }
        return data

    def __getattr__(self, name):
        value = self.data.get(name, None)
        if value is None:
            raise AttributeError(name)
        return value

    def _get_field_value(self, name):
        field = self._get_field(name)
        value = None
        if field:
            value = self._custom_fields_value.get(field, None)
            if value is None:
                value = CustomFieldValueRequest(self.contact_request.api_request).get(field)
        return value

    def _get_field(self, name):
        if not self._custom_fields:
            self._get_custom_fields()
        return self._custom_fields.get(name, None)

    def _get_custom_fields(self):
        self._custom_fields = CustomFieldsRequest(self.contact_request.api_request).get_all()['fields']


class ContactRequest(AbstractRequest):
    def __init__(self, api_request):
        super(ContactRequest, self).__init__(api_request)
        self.endpoint = API_ENDPOINT.CONTACT

    def create(self):
        return self.api_request.post_request(endpoint=self.endpoint)

    def create_or_update(self, params):
        return self.api_request.post_request(
            endpoint=self.endpoint + '/sync',
            params=params)

    def lists(self, params):
        return self.api_request.post_request(
            endpoint=self.endpoint + '/contactLists',
            payload=params)

    def automations(self, message_id):
        return self.api_request.post_request(
            endpoint=self.endpoint + '/' + message_id + '/contactAutomations')

    def get_all_to_object(self, params=None):
        if not params:
            params = {"status": "-1", "orders[email]": "ASC"}
        return self.from_dict_to_objects(
            self.api_request.get_request(
                endpoint=self.endpoint,
                params=params))

    def get_all(self, params=None):
        if not params:
            params = {"status": "-1", "orders[email]": "ASC"}
        return self.api_request.get_request(
            endpoint=self.endpoint,
            params=params)

    def from_dict_to_objects(self, data):
        contacts = []
        if 'meta' in data:
            self.meta_data = data['meta']
        if 'contacts' in data and isinstance(data['contacts'], list):
            for contact_data in data['contacts']:
                contacts.append(Contact(self).from_dict(contact_data))
        return contacts




class AbstractRequest(object):
    def __init__(self, api_request):
        self.api_request = api_request
        self.endpoint = ''

    def create(self, json_data):
        return self.api_request.post_request(endpoint=self.endpoint, json_data=json_data)

    def get_all(self):
        return self.api_request.get_request(endpoint=self.endpoint)

    def get(self, object_id=False):
        if not object_id:
            return self.get_all()
        return self.api_request.get_request(endpoint=self.endpoint + '/' + str(object_id))

    def update(self, object_id, json_data=None):
        return self.api_request.put_request(endpoint=self.endpoint + '/' + str(object_id), json_data=json_data)

    def delete(self, object_id):
        return self.api_request.delete_request(endpoint=self.endpoint + '/' + str(object_id))



import logging

from activecampaign_api.api.configuration import Configuration
from activecampaign_api.api.consts import USER_AGENT
from activecampaign_api.api.api_request import ApiRequest
from activecampaign_api.api.message import MessageRequest
from activecampaign_api.api.contact import ContactRequest
from activecampaign_api.api.segment import SegmentRequest
from activecampaign_api.api.custom_field import CustomFieldsRequest
from activecampaign_api.api.custom_field_value import CustomFieldValueRequest
from activecampaign_api.api.email_activities import EmailActivitiesRequest


class Api(object):
    def __init__(self, api_url, api_token):
        self._logger = logging.getLogger('activecampaign.api')
        self.api_request = ApiRequest(self._logger, Configuration(api_url, api_token), USER_AGENT)

    @property
    def message(self):
        return MessageRequest(self.api_request)

    @property
    def contact(self):
        return ContactRequest(self.api_request)

    @property
    def segment(self):
        return SegmentRequest(self.api_request)

    @property
    def custom_field(self):
        return CustomFieldsRequest(self.api_request)

    @property
    def custom_field_value(self):
        return CustomFieldValueRequest(self.api_request)

    @property
    def email_activities(self):
        return EmailActivitiesRequest(self.api_request)




from setuptools import setup, find_packages
from activecampaign_api.version import __version__

setup(
    name='activecampaign_api',
    version=__version__,
    packages=find_packages(),
    url='',
    license='',
    author='Yevhen Pechurin',
    author_email='yevhen.pechurin@gmail.com',
    description='Api for activecampaign'
)



class Configuration(object):
    def __init__(self, api_url, api_token):
        self._api_token = api_token
        self._api_url = api_url + '/api/3'

    @property
    def api_url(self):
        return self._api_url

    @property
    def api_token(self):
        return self._api_token




from activecampaign_api.api.abstract_request import AbstractRequest
from activecampaign_api.api.consts import API_ENDPOINT


class CustomFieldsRequest(AbstractRequest):
    def __init__(self, api_request):
        super(CustomFieldsRequest, self).__init__(api_request)
        self.endpoint = API_ENDPOINT.FIELD
